function [B,C,D,solved,err] = sudoku_excl_func(A)
%Exclusion principle solution of Sudokus.
%Input: matrix A of initial values, Output: solution matrix D.

% Copyright (C) 2007 Felix Voigt.
% For full notice see file "sudoku_func.m".

B = zeros(9,9,9);
%B(x,y,z)=0(1) means that A(x,y) = z is (im)possible.
%B(x,y,z) = 2 means A(x,y) = z.
C = zeros(9,9);
%Number of excluded digit at point (x,y).
%C(x,y) = 10 means that A(x,y) is known.
D = zeros(9,9); %Solution.
D = A;
solved = 0; %Flag signalizes, whether Sudoku is solved.
err = 0; %Error flag.

%Looks at every point (x,y), whether a digit is known there,
%If this is the case,
%this digit is excluded at every point of the corresponding
%column, row and 3x3 square.
for x = 1:9
    for y = 1:9
        if D(x,y) > 0
        [B,C,D] = exclude(x,y,B,C,D);
        end %if
    end %y
end %x

cont = 1;
while cont == 1 %Perform procedures as long as they effect a change of D.
    D_alt = D;
    
    %Procedure according to "first" exclusion principle.
    %Checks at every point (x,y), whether there are excluded 8 digits.
    %If this is the case, the left 9th digit is entered in D(x,y).
    for x = 1:9
        for y = 1:9
            if C(x,y) == 8
                for z = 1:9
                    if B(x,y,z) == 0
                        D(x,y) = z;
                        [B,C,D] = exclude(x,y,B,C,D);
                    end %if
                end %for
            end %if
            if C(x,y) == 9
                err = 1; %Sudoku is not solvable.
            end %if
        end %y
    end %x
    
    %Procedure according to "second" exclusion principle:
    %"Somewhere in a column, row, a 3x3 square the digit z must be located."

    %Remark: The "first" and "second" exclusion principle are complementary
    %to each other.
    %"first":       8 digits are excluded at point (x,y).
    %               -> At point (x,y) the 9th digit must stand.
    %"second": At 8 points of a column, row, a 3x3 square the digit z is excluded.
    %               ->Digit z must be located at the 9th point. 
    for z = 1:9 %Loop for all 9 digits.
        %Columns.
        for y = 1:9
            addition = 0;
            for x = 1:9
                addition = addition + B(x,y,z);
            end %x
            if addition == 8 %Digit z must be located at the point of the column, where B(x,y,z) = 0.
                for x = 1:9 %Searches for the right place in column for digit z.
                    if B(x,y,z) == 0
                        D(x,y) = z;
                        [B,C,D] = exclude(x,y,B,C,D);
                    end %if
                end %x
            end %if
            if addition == 9
                err = 1; %Sudoku is not solvable.
            end %if
        end %y
        %Rows.
        for x = 1:9
            addition = 0;
            for y = 1:9
                addition = addition + B(x,y,z);
            end %y
            if addition == 8 %"Row code" analogue to code for exclusion in a column. 
                for y = 1:9
                    if B(x,y,z) == 0
                        D(x,y) = z;
                        [B,C,D] = exclude(x,y,B,C,D);
                    end %if
                end %y
            end %if
            if addition == 9
                err = 1; %Sudoku is not solvable.
            end %if
        end %x
        %3x3 square.
        for kx = 0:2 %3x3 square indecees.
            for ky = 0:2
                addition = 0;
                for x = 1:3
                    for y = 1:3
                        addition = addition + B(3*kx+x,3*ky+y,z);
                    end %y
                end %x
                if addition == 8 %"3x3 square" code analogue to code for exclusion in a column and exclusion in a row.
                    for x = 1:3
                        for y = 1:3
                            if B(3*kx+x,3*ky+y,z) == 0
                                D(3*kx+x,3*ky+y) = z;
                                [B,C,D] = exclude(3*kx+x,3*ky+y,B,C,D);
                            end %if
                        end %y
                    end %x
                end %if
                if addition == 9
                    err = 1; %Sudoku is not solvable.
                end %if
            end %ky
        end %kx
    end %z
        
    if D == D_alt
        cont =0;
    end %if
end %while

if sum(sum(C)) == 810 %If Sudoku is solved.
    solved =1;
end %if
    
    
    